/*
 * Copyright (c) 2006 The Trustees of Indiana University and Indiana
 *                    University Research and Technology
 *                    Corporation.  All rights reserved.
 * Copyright (c) 2006 The Technical University of Chemnitz. All 
 *                    rights reserved.
 *
 * Author(s): (alphabetically)
 *            Sven Eckelmann
 *            Torsten Hoefler 
 *            Mirko Reinhard 
 *            Matthias Treydte 
 */

#ifndef AF_ENET_COMPAT_H_
#define AF_ENET_COMPAT_H_

#ifndef __acquires
#define __acquires(x)
#endif

#ifndef __releases
#define __releases(x)
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)

#ifndef WARN_ON
#define WARN_ON(x) do { \
   if (unlikely(!!(x))) { \
           printk(KERN_ERR "KERNEL: assertion (%s) failed at %s (%d)\n", \
                   #x,  __FILE__ , __LINE__); \
   } \
} while(0)
#endif /* WARN_ON */

#endif /* < KERNEL_VERSION(2,6,0) */


#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,13)

#define skb_get_timestamp(_skb, _stamp); \
	do { _stamp = (_sk)->stamp; } while (0)

#endif /* < KERNEL_VERSION(2,6,13) */

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,21)

#define register_sysctl_table(_sysctl_table) \
	register_sysctl_table((_sysctl_table), 0)

#endif /* < KERNEL_VERSION(2,6,21) */


#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,22)

#define enet_write_queue_purge(_sk); \
	do { sk_stream_writequeue_purge(_sk); } while (0)

#define skb_reset_network_header(_skb); \
	do { (_skb)->nh.raw = (_skb)->data; } while (0)

#define skb_mac_header(_skb) \
	((_skb)->mac.raw)

#define skb_network_header(_skb) \
	((_skb)->nh.raw)

#define skb_mac_header(_skb) \
	((_skb)->mac.raw)

#endif /* < KERNEL_VERSION(2,6,22) */

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,24) && \
    LINUX_VERSION_CODE > KERNEL_VERSION(2,6,11)

#define sk_alloc(_etype, _priority, _proto) \
	sk_alloc(_etype, _priority, _proto, 1)

#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,24)

#define dev_parse_header(_skb, _addr) \
	dev->hard_header_parse(_skb, _addr)

#define HAS_HARD_HEADER(_dev) \
	((_dev)->hard_header)

#define HAS_HARD_HEADER_PARSE(_dev) \
	((_dev)->hard_header_parse)

#define NET_PARAM

#define NET_ARG

#define INIT_NET_ARG

#define dev_hard_header(_skb, _dev, _type, _daddr, _saddr, _len) \
	(_dev)->hard_header(_skb, _dev, _type, _daddr, _saddr, _len)

#else

#define HAS_HARD_HEADER(_dev) \
	1

#define HAS_HARD_HEADER_PARSE(_dev) \
	1

#define NET_PARAM \
	struct net *net,

#define NET_ARG \
	net,

#define INIT_NET_ARG \
	&init_net,

#endif /* < KERNEL_VERSION(2,6,24) */


#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,25)

#define sk_wmem_free_skb(_sk, _skb) \
	sk_stream_free_skb(_sk, _skb)

#define sk_mem_reclaim(_sk) \
	sk_stream_mem_reclaim(_sk)

#endif /* < KERNEL_VERSION(2,6,25) */


/* EDP header access */
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,22)
#define EDP_HDR(skb) ((struct edp_header *) skb_network_header(skb))
#define ESP_HDR(skb) ((struct esp_header *) skb_network_header(skb))
#define EDP_MAC_HDR(skb) (skb)->mac_header
#else
#define EDP_HDR(skb) ((struct edp_header *) skb->nh.raw)
#define ESP_HDR(skb) ((struct esp_header *) skb->nh.raw)
#define EDP_MAC_HDR(skb) (skb)->mac.raw
#endif

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,21)
#define SKB_INPUT_DEV(_skb) ((_skb)->dev)
#else
#define SKB_INPUT_DEV(_skb) ((_skb)->input_dev)
#endif

#endif /* AF_ENET_COMPAT_H_ */
