/*
 * Copyright (c) 2006 The Trustees of Indiana University and Indiana
 *                    University Research and Technology
 *                    Corporation.  All rights reserved.
 * Copyright (c) 2006 The Technical University of Chemnitz. All 
 *                    rights reserved.
 *
 * Author(s): (alphabetically)
 *            Sven Eckelmann
 *            Torsten Hoefler 
 *            Mirko Reinhard 
 *            Matthias Treydte 
 */

#ifndef ENET_STREAM_STATS_H_
#define ENET_STREAM_STATS_H_

#include <net/sock.h>

/**
 * For iterating a hashtable
 */
struct htable_iterator {
	/** the bucket of the htable */
	int bucket;
	/** the socket within this bucket */
	struct sock *sk;
};

/**
 * Initializes the iterator to the before-first position.
 */
#define ITER_INIT(it) do { it->sk = NULL; it->bucket = -1; } while (0)

/**
 * Check if it points to a valid htable_iterator.
 */
#define ITER_VALID(it) ((it->sk != NULL) && (it->bucket >= 0))

/**
 * Check if the iterator is on the before-first position.
 */
#define ITER_BEFORE_FIRST(it) (it->bucket < 0)

#endif
